﻿/*
* This file is part of VECTO.
*
* Copyright © 2012-2019 European Union
*
* Developed by Graz University of Technology,
*              Institute of Internal Combustion Engines and Thermodynamics,
*              Institute of Technical Informatics
*
* VECTO is licensed under the EUPL, Version 1.1 or - as soon they will be approved
* by the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use VECTO except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/community/eupl/og_page/eupl
*
* Unless required by applicable law or agreed to in writing, VECTO
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* Authors:
*   Stefan Hausberger, hausberger@ivt.tugraz.at, IVT, Graz University of Technology
*   Christian Kreiner, christian.kreiner@tugraz.at, ITI, Graz University of Technology
*   Michael Krisper, michael.krisper@tugraz.at, ITI, Graz University of Technology
*   Raphael Luz, luz@ivt.tugraz.at, IVT, Graz University of Technology
*   Markus Quaritsch, markus.quaritsch@tugraz.at, IVT, Graz University of Technology
*   Martin Rexeis, rexeis@ivt.tugraz.at, IVT, Graz University of Technology
*/

using TUGraz.VectoCommon.Models;
using TUGraz.VectoCommon.Utils;
using TUGraz.VectoCore.Models.Connector.Ports;

namespace TUGraz.VectoCore.Models.SimulationComponent.Impl {
	public class TorqueConverterWrapper
	{
		protected internal CycleTorqueConverter CycleTorqueConverter;
		protected internal TorqueConverter TorqueConverter;
		protected bool UseCycleTorqueConverter;
		
		public TorqueConverterWrapper(bool useCycle, CycleTorqueConverter cycleTorqueConverter, TorqueConverter torqueConverter)
		{
			UseCycleTorqueConverter = useCycle;
			if (useCycle) {
				CycleTorqueConverter = cycleTorqueConverter;
			} else {
				TorqueConverter = torqueConverter;
			}
		}

		public ITnOutPort NextComponent
		{
			set {
				if (UseCycleTorqueConverter) {
					CycleTorqueConverter.NextComponent = value;
				} else {
					TorqueConverter.NextComponent = value;
				}
			}
		}

		public IResponse Initialize(NewtonMeter outTorque, PerSecond outAngularVelocity, PerSecond inAngularVelocity)
		{
			return UseCycleTorqueConverter
				? CycleTorqueConverter.Initialize(outTorque, outAngularVelocity, inAngularVelocity)
				: TorqueConverter.Initialize(outTorque, outAngularVelocity);
		}

		public IResponse Request(Second absTime, Second dt, NewtonMeter outTorque, PerSecond outAngularVelocity, PerSecond inAngularVelocity, bool dryRun = false)
		{
			return UseCycleTorqueConverter
				? CycleTorqueConverter.Request(absTime, dt, outTorque, outAngularVelocity, inAngularVelocity, dryRun)
				: TorqueConverter.Request(absTime, dt, outTorque, outAngularVelocity, dryRun);
		}

		public void Locked(NewtonMeter outTorque, PerSecond outAngularVelocity, NewtonMeter inTorque, PerSecond inAngularVelocity)
		{
			if (UseCycleTorqueConverter) {
				CycleTorqueConverter.Locked(outTorque, outAngularVelocity, inTorque, inAngularVelocity);
			} else {
				TorqueConverter.Locked(outTorque, outAngularVelocity, inTorque, inAngularVelocity);
			}

		}
	}
}